/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: pixel.h
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#ifndef PIXEL_H
#define PIXEL_H
//============================================================================
//	Pixel :
//		Dfinie un lment de la piste.
//		Fait la correspondance entre coordonnes de la piste et les
//		coordonnes de la widgets qui le dessinera.
//
//	Seule la class amie WidgetTrack peut crer un Pixel.
//	Un pixel n'est pas copiable.
//
//============================================================================

#include "shapepixel.h"

#include <cstdlib>	// pour abort

class WidgetTrack;
class Pixel 
{
public:
	static const char asciiTrack = ' ';

	int					num()		const;
	const QString&		nameColor()	const;
	const QColor&		color()		const;
	char				ascii()		const;
	bool				track()		const;

	//	Coordonnes en pixel de la carte.
	int					x()			const;
	int					y()			const;


private:
	friend WidgetTrack;

	Pixel();

	//	Coordonnes relles dans WidgetTrack.
	int					coordX()	const;
	int					coordY()	const;

	void				setNum( int num );
	void				set( int x, int y );
	void				setCoords( int coordX, int coordY );
	void				setShape( const ShapePixel& shape );

	const ShapePixel&	shape()		const;

private:
	Pixel( const Pixel& ) { std::abort(); }
	void operator=( const Pixel& ) { std::abort(); }

private:
	int					num_;
	const ShapePixel*	shape_;
	int					x_;
	int					y_;

	int					coordX_;
	int					coordY_;
};

//	oprateurs
inline bool operator==( const Pixel& left, const Pixel& right )
{
	// La comparaison su le n est suffisante car la class Pixel
	// reprsente physiquement les pixels de la piste, donc un
	// n de pixel existe une fois pour des coordonnes fixes.
	return left.num() == right.num();
}

inline bool operator!=( const Pixel& left, const Pixel& right )
{
	return ! operator==( left, right );
}

//	Fonctions inlines
inline Pixel::Pixel()
:	 num_(-1)
	,shape_(NULL)
{}

inline void	Pixel::setNum( int num ) { num_ = num; }

inline void Pixel::set( int x, int y )
{
	x_		= x;
	y_		= y;
}

inline void Pixel::setCoords( int coordX, int coordY )
{
	coordX_	= coordX;
	coordY_	= coordY;
}

inline void Pixel::setShape( const ShapePixel& shape )
{
	shape_ = &shape;
}

inline int				Pixel::num()		const { return num_; }

inline const QString&	Pixel::nameColor()	const { return shape_->name(); }

inline int				Pixel::x()			const { return x_; }
inline int				Pixel::y()			const { return y_; }

inline const QColor&	Pixel::color()		const { return shape_->color(); }
inline char				Pixel::ascii()		const { return shape_->ascii(); }
inline bool				Pixel::track()		const
{
	return shape_->ascii() == asciiTrack;
}

inline int				Pixel::coordX()		const { return coordX_; }
inline int				Pixel::coordY()		const { return coordY_; }

inline const ShapePixel& Pixel::shape()		const { return *shape_; }

#endif	// PIXEL_H

