/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: script.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "script.h"

#include <cstdlib>	//system
#include <cstring>	//strerror
#include <cerrno>	//errno
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>

const QString	Script::logFile("/tmp/script_exec.log");
Script::Script	Script::script_;

Script::Script()
:	 on_(false)
	,pid_(-1)
	,exitCode_(0)
{
	signal( SIGCHLD, onSigChild );
}

Script::~Script()
{
	if ( pid_ != -1 )
		kill();
}

void Script::onSigChild( int )
{
	int status;

	int wpErr;
	while ( (wpErr=waitpid( script_.pid_, &status, WNOHANG | 0 )) == -1 
														&& errno == EINTR ) {}

	script_.end( wpErr == -1 ? -1 : status );
}

bool Script::exec()
{
	const QString	cmd	= name_ + ' ' + params_ + " 2> " + logFile;

	switch ( pid_ = fork() )
	{
	case -1:
		exitCode_ = -1;
		msgError_ = QString("Erreur au lancement : ") + strerror(errno);
		return false;
	
	case 0: // fils
		{
			char* argv[] =
			{
				 "sh"
				,"-c"
				,const_cast<char*>(cmd.latin1())
				,0
			};
			execv( "/bin/sh", argv );
			exit (127);
		}
	
	default:
		msgError_ = QString::null;
		return true;
	}
}

void Script::end( int status /*-1 error sys*/ )
{
	pid_ = -1;
	if ( status == -1 )
	{
		exitCode_ = -1;
		msgError_ = QString("Erreur systme : ") + strerror(errno);
	}
	else if ( WIFEXITED(status) )
	{
		exitCode_ = WEXITSTATUS(status);
	}
	else
	{
		exitCode_ = -1;
		msgError_ = "Arrt du fils sur le signal "
					+ QString::number(WTERMSIG(status))
				;
	}
}

void Script::kill()
{
	::kill( pid_, 9 );
}


