/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: trackeditor.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "trackeditor.h"
#include "dialogselectarea.h"

#include "pixel/pixel.h"
#include "pixel/outputpixel.h"
#include "pixel/listshapepixel.h"

#include "widgettrack/widgettrack.h"

#include "config/config.h"

#include "divers/outputpoint.h"
#include "divers/ext.h"
#include "divers/bmpascii.h"

#include "wdivers/dialogpalettecolor.h"

#include "icons/icons.h"

#include <cassert>

#include <qmenubar.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qfileinfo.h>

namespace
{
	const QString GB_btnUndo( "&Dfaire" );
	const QString GB_btnRedo( "&Refaire" );

	inline void
	makeLabelBtn( QPushButton* pb, const QString& label, uint count )
	{
		pb->setText( label + " : " + QString::number(count) );
		pb->setDisabled( count == 0 );
	}

	inline bool
	borderTrack( const Pixel& pixel )
	{
		return	pixel.x() == 0 || pixel.x() == Dim::maxX - 1
			||	pixel.y() == 0 || pixel.y() == Dim::maxY - 1
			;
	}

	QToolButton*
	newToolButton( const QPixmap& icon, int accel,	const QString& label,
															QWidget* parent )
	{
		QToolButton* tb = new QToolButton( parent );
		tb->setAccel( accel );
		tb->setIconSet( icon );
		tb->setTextLabel( label );

		return tb;
	}
};

TrackEditor::TrackEditor( QWidget* parent, const char* name )
:	 TrackMainWindow( Edit, parent, name )
	,action_(no)
	,mouseLeftPressPixel_(NULL)
	,dialogSelectArea_(NULL)
	,activeColor_(Qt::darkGray)
{
	setIconText( "diteur: " + iconText() );

	const Icons& icons = Icons::instance();
	//	--- 
	//	Bouttons permettant de charger/enregistrer une piste
	//	---
	QPushButton* btnLoad =
			new QPushButton( icons.fileOpen(), "&Charger", parentTopBar() );
	addTopBar( btnLoad );
	connect( btnLoad, SIGNAL(clicked()), SLOT(dialogLoad()) );

	btnSave_ = new QPushButton( icons.fileSave(), "&Sauver", parentTopBar() );
	addTopBar( btnSave_ );
	connect( btnSave_, SIGNAL(clicked()), SLOT(save()) );
	btnSave_->setDisabled( true );

	QPushButton* btnSave2 = new QPushButton( "Sauver sous", parentTopBar() );
	addTopBar( btnSave2 );
	connect( btnSave2, SIGNAL(clicked()), SLOT(saveDialog()) );

	addSeparatorTopBar();

	//	---
	//	Bouttons undo/redo
	//	---
#define BTN_UR(BTN,ICON,LABEL,PARENT,SLOT_NAME)			\
	BTN = new QPushButton( ICON, LABEL, PARENT );		\
	addTopBar( BTN );									\
	connect( BTN, SIGNAL(clicked()), SLOT(SLOT_NAME) );	\
	BTN->setDisabled( true );							\
	BTN->setAutoRepeat( true )

	BTN_UR( btnUndo_, icons.undo(), GB_btnUndo, parentTopBar(), undo() );
	BTN_UR( btnRedo_, icons.redo(), GB_btnRedo, parentTopBar(), redo() );
	
	addSeparatorTopBar();

	//	---
	//	
	//	---
	QToolButton* btnPreview = newToolButton( icons.preview(),
										ALT+Key_A, "Apercu", parentTopBar() );
	btnPreview->setToggleButton( true );
	addTopBar( btnPreview );
	connect( btnPreview, SIGNAL(toggled(bool)), SLOT(preview(bool)) );

	QToolButton* btnDraw = newToolButton( icons.refresh(),
									ALT+Key_R, "Rafrachir", parentTopBar() );
	addTopBar( btnDraw );
	connect( btnDraw, SIGNAL(clicked()), SLOT(redraw()) );

	QToolButton* btnAdjust = newToolButton( icons.correct(),
									ALT+Key_G, "Corriger", parentTopBar() );
	addTopBar( btnAdjust );
	connect( btnAdjust, SIGNAL(clicked()), SLOT(adjust()) );

	addSeparatorTopBar();

	//	---
	//	
	//	---
	QToolButton* btnQuit = newToolButton( icons.exit(),
										ALT+Key_F, "Fermer", parentTopBar() );
	addTopBar( btnQuit );
	connect( btnQuit, SIGNAL(clicked()), SLOT(close()) );

	//	----
	//	Connection des signaux de widgetTrack_.
	//	----
	connect( widgetTrack_, SIGNAL(mouseLeftPressPixel(QMouseEvent*, Pixel&)),
							SLOT(mouseLeftPressPixel(QMouseEvent*, Pixel&)) );
	connect( widgetTrack_, SIGNAL(mouseMouveOnPixel(Pixel&)),
											SLOT(mouseMouveOnPixel(Pixel&)) );
	connect( widgetTrack_, SIGNAL(mouseLeftReleasePixel(QMouseEvent*,Pixel&)),
							SLOT(mouseLeftReleasePixel(QMouseEvent*,Pixel&)) );
	connect( widgetTrack_, SIGNAL(edited()), SLOT(edited()) );
	//	----


	//	---
	//	Dialog de la palette de couleurs.
	//	---
	dialogPaletteColor_ = new DialogPaletteColor(this);
	connect( dialogPaletteColor_, SIGNAL(colorChanged(const QColor&)),
									SLOT(activeColorChanged(const QColor&)) );

	dialogPaletteColor_->move(
						config_.paletteColorX(), config_.paletteColorY() );
}

void TrackEditor::show()
{
	TrackMainWindow::show();
	dialogPaletteColor_->show();
}

void TrackEditor::preview( bool pview )
{
	//	Si on passe en mode preview il n'est pas ncessaire d'effacer l'cran.
	widgetTrack_->setWidthPen( pview ? Dim::gap*2 : 0 );
	widgetTrack_->repaint( ! pview );
}

void TrackEditor::redraw()
{
	widgetTrack_->repaint( true );
}

void TrackEditor::adjust()
{
	const bool correction = widgetTrack_->normalizeTrackBorder()
						&	widgetTrack_->checkStarterArea()
						;
	statusBar_.message(
			correction ? "La piste a t corrige." : "Pas de correction." );
}

void TrackEditor::edited()
{
	btnSave_->setDisabled( false );
	setCaption( "* " + caption() );
	if ( ! readOnly_ )
		emit edited( trackName_, Modified );
}

void TrackEditor::undo()
{
	widgetTrack_->play( trackImageManager_.topBefore(),
											trackImageManager_.makeAfter() );
	trackImageManager_.popBefore();

	makeLabelBtn( btnRedo_, GB_btnRedo, trackImageManager_.countAfter() );
	makeLabelBtn( btnUndo_, GB_btnUndo, trackImageManager_.countBefore() );
}

void TrackEditor::redo()
{
	widgetTrack_->play( trackImageManager_.topAfter(),
											trackImageManager_.makeBefore() );
	trackImageManager_.popAfter();

	makeLabelBtn( btnRedo_, GB_btnRedo, trackImageManager_.countAfter() );
	makeLabelBtn( btnUndo_, GB_btnUndo, trackImageManager_.countBefore() );
}

void TrackEditor::mouseLeftPressPixel( QMouseEvent* me, Pixel& pixel )
{
	statusSelection_ = outPutPixel( pixel );

	if ( ! dialogSelectArea_ )
	{
		switch ( me->button() )
		{
		case LeftButton:
			mouseLeftPressPixel_	= &pixel;
			if ( me->state() & ShiftButton )
			{
				if ( ! borderTrack( pixel ) )
				{
					dialogSelectAreaColorize( pixel.num(),
													pixel.num()-Dim::maxX-1 );
					setCursor( pointingHandCursor );
					action_ = drawingBrush;
				}
			}
			else
			{
				setCursor( CrossCursor );
				action_ = drawing;
			}
			break;

		}
	}

	statusBar_.message( statusSelection_ );
}

void TrackEditor::mouseMouveOnPixel( Pixel& pixel )
{
	switch ( action_ )
	{
	case drawing:
		statusBar_.message( statusSelection_ 
						+ " : move " + outPutPoint( pixel.x(), pixel.y() ) );

		widgetTrack_->select( activeColor_, *mouseLeftPressPixel_, pixel );
		break;
	
	case drawingBrush:
		if ( ! borderTrack( pixel ) )
			dialogSelectAreaColorize( pixel.num(), pixel.num()-Dim::maxX-1 );
	}
}

void TrackEditor::mouseLeftReleasePixel( QMouseEvent* me, Pixel& pixel )
{
	switch ( me->button() )
	{
	case LeftButton:
		setCursor( ArrowCursor );
		if ( action_ == drawing )
		{
			if ( *mouseLeftPressPixel_ != pixel )
				statusSelection_ += " au " + outPutPixel( pixel );

			statusBar_.message( "Colorisation du " + statusSelection_ );

			fillSelection( pixel );
		}
		action_ = no;
		break;
	
	case MidButton:
		if ( ! dialogSelectArea_ )
		{
			dialogSelectArea_ = new DialogSelectArea( this, 0 );
			connect( dialogSelectArea_, SIGNAL(areaChanged(int,int)),
									SLOT(dialogSelectAreaChanged(int,int)) );
			connect( dialogSelectArea_, SIGNAL(paintArea(int,int)),
									SLOT(dialogSelectAreaColorize(int,int)) );
			connect( dialogSelectArea_, SIGNAL(close()),
											SLOT(dialogSelectAreaClosed()) );
			dialogSelectArea_->show();
			dialogSelectArea_->initStarterArea();
		}
		break;

	case RightButton:
		activeColor_ = pixel.color();
		dialogPaletteColor_->setColor( activeColor_ );
		statusBar_.message( statusSelection_ +
						" La couleur par dfaut est " + pixel.nameColor() );
		break;
	}
}

void TrackEditor::dialogSelectAreaChanged( int np0, int np1 )
{
	widgetTrack_->select( activeColor_, np0, np1 );
}

void TrackEditor::dialogSelectAreaColorize( int np0, int np1 )
{
	widgetTrack_->cancelCleanSelect();

	TrackPixmap* before = widgetTrack_->fill( activeColor_, np0, np1 );
	trackImageManager_.pushBefore( before );

	makeLabelBtn( btnRedo_, GB_btnRedo, trackImageManager_.countAfter() );
	makeLabelBtn( btnUndo_, GB_btnUndo, trackImageManager_.countBefore() );
}

void TrackEditor::dialogSelectAreaClosed()
{
	delete dialogSelectArea_;
	dialogSelectArea_ = NULL;
	widgetTrack_->cleanSelect();
}

void TrackEditor::activeColorChanged( const QColor& c )
{
	activeColor_ = c;
	if ( dialogSelectArea_ )
		widgetTrack_->select( c, dialogSelectArea_->topLeftPixel(),
									dialogSelectArea_->bottomRightPixel() );
}

void TrackEditor::fillSelection( Pixel& releasePixel )
{
	widgetTrack_->cancelCleanSelect();

	TrackPixmap* before = widgetTrack_->fill( activeColor_,
								mouseLeftPressPixel_, &releasePixel );

	trackImageManager_.pushBefore( before );
	makeLabelBtn( btnRedo_, GB_btnRedo, trackImageManager_.countAfter() );
	makeLabelBtn( btnUndo_, GB_btnUndo, trackImageManager_.countBefore() );
}

bool TrackEditor::save()
{
	if ( widgetTrack_->save( trackName_ ) )
	{
		readOnly_ = false;	// L'utilisateur peut avoir chang les perms.
		setCaption( trackName_ );
		btnSave_->setDisabled( true );
		statusBar_.message( "Piste sauve dans " + trackName_ );
		emit edited( trackName_, Saved );
		return true;
	}
	else
	{
		statusBar_.message( "chec sauvegarde de " + trackName_ );
		btnSave_->setDisabled( false );
		return false;
	}
}

bool TrackEditor::saveDialog()
{
	QString file =
		QFileDialog::getSaveFileName( config_.pathTracks(), 
												"Pistes (*."+ext::track+")" );
	if ( ! file.isEmpty() )
	{
		if ( file.mid( file.length() - 3 ) != ext::track )
			file += '.' + ext::track;

		trackName_ = file;
		return save();
	}

	return false;
}

bool TrackEditor::testSaveDialog()
{
	if ( widgetTrack_->state() == WidgetTrack::Changed )
	{
		const int r =
			QMessageBox::warning( this, "SAUVER",
						QString( "La piste %1 est modifie, enregistrer ?")
						.arg(trackName_), "Oui", "Non", "Annuler" );
		switch( r )
		{
		case 0:	// Oui
			return (readOnly_ || trackName_.isEmpty()) ? saveDialog() : save();

		case 1:	// Non
			emit edited( trackName_, Closed );
			return true;

		case 2:	// Annuler
		default:
			return false;
		}
	}

	return true;
}

void TrackEditor::load( const QString& fileTrack )
{
	if ( widgetTrack_->load( fileTrack ) )
	{
		trackName_ = fileTrack;
		setCaption( fileTrack );

		readOnly_ = ! QFileInfo( fileTrack ).isWritable();
		if ( readOnly_ )
			setCaption( caption() + " EN LECTURE SEUL" );
		
		if ( widgetTrack_->state() != WidgetTrack::Saved )
			emit edited();
	}

	makeLabelBtn( btnRedo_, GB_btnRedo, 0 );
	makeLabelBtn( btnUndo_, GB_btnUndo, 0 );
	btnSave_->setDisabled( widgetTrack_->state() != WidgetTrack::Changed );
}

void TrackEditor::dialogLoad()
{
	const QString filters = "Pistes (*."+ext::track + " *.bmp)";
	if ( testSaveDialog() )
	{
		const QString file =
			QFileDialog::getOpenFileName( config_.pathTracks(),
										filters, NULL, 0, "Charger Piste" );

		if ( ! file.isEmpty() )
		{
			if ( file.contains( ".bmp", false ) != 0 )
			{
				QString cirName;
				if ( Convert::bmp2cir( file, &cirName ) )
					load( cirName );
				else
					warning( true,	"Impossible de convertir " + file +
														" -> " + cirName );
			}
			else
				load( file );
		}
	}
}

void TrackEditor::closeEvent( QCloseEvent* ce )
{
	config_.setTrackEditorXY( x(), y() );
	config_.setPaletteColorXY(
						dialogPaletteColor_->x(), dialogPaletteColor_->y() );
	if ( testSaveDialog() )
		ce->accept();
	else
		ce->ignore();
}

