/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: trackmainwindow.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "trackmainwindow.h"
#include "widgettrack/widgettrack.h"

#include "config/config.h"
#include "icons/icons.h"

#include <qlayout.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qaccel.h>

namespace
{
	const QString _nottitle_( "PAS DE CIRCUIT" );
};

TrackMainWindow::TrackMainWindow( Type config, QWidget* parent,
															const char* name )
:	 QMainWindow( parent, name, WType_TopLevel | WDestructiveClose )
	,config_(Config::instance())
	,statusBar_(*statusBar())
{
	setIconText( "F1 Racing" );
	setIcon( Icons::instance().simulator() );
	setCaption( _nottitle_ );
	statusBar_.setSizeGripEnabled( false );

	QAccel* accel = new QAccel( this );
	accel->connectItem( accel->insertItem(CTRL+Key_F1), this, SIGNAL(help()) );

	centralWidget_	= new QWidget( this ); 
	setCentralWidget( centralWidget_ );

	QVBoxLayout*	verticalLayout	= new QVBoxLayout( centralWidget_ );

	topFrame_ = new QFrame( centralWidget_ );
	topFrame_->setFrameShape( QFrame::StyledPanel );
    topFrame_->setFrameShadow( QFrame::Raised );
    topLayout_ = new QHBoxLayout( topFrame_, 4 ); 
	topLayout_->setMargin( 2 );
	verticalLayout->addWidget( topFrame_, 0, 0 );

	widgetTrack_ = 
		new WidgetTrack( config == TrackSimulator ? Dim::gap*2 : 0
													,centralWidget_, name );
	verticalLayout->addWidget( widgetTrack_, 1, 0 );

	connect( widgetTrack_, SIGNAL(warning(bool,const QString&)),
									this, SLOT(warning(bool,const QString&)) );
}

void TrackMainWindow::polish()
{
	QMainWindow::polish();
	setFixedSize( sizeHint() );
}

void TrackMainWindow::addTopBar( QWidget* add )
{
	topLayout_->addWidget( add );
}

void TrackMainWindow::addTopBar( QLayout* add )
{
	topLayout_->addLayout( add );
}

void TrackMainWindow::addSeparatorTopBar()
{
	topLayout_->addItem( new QSpacerItem( 10, 10 ) );
}

void TrackMainWindow::warning( bool error, const QString& msg )
{
	if ( error )
		QMessageBox::critical( this, "Erreur", msg );
	else
		QMessageBox::warning( this, "Alerte", msg );
}

