/*****************************************************************************
 * F1 Racing diteur/simulateur
 * Fichier: tracksimulator.cpp
 *
 * (c) Leroux philippe, 2003
 *
 *
 * This program is distributed under the terms of the GNU General Public
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/
#include "tracksimulator.h"
#include "dialogresults.h"
#include "dialogpauseon.h"
#include "log.h"

#include "script/script.h"
#include "script/dialogscript.h"

#include "widgettrack/dimensions.h"
#include "widgettrack/widgettrack.h"
#include "config/config.h"
#include "divers/ext.h"
#include "divers/outputpoint.h"
#include "pixel/outputpixel.h"
#include "icons/icons.h"

#include <qapplication.h>	// qApp

#include <qmenubar.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qcheckbox.h>
#include <qstatusbar.h>
#include <qlayout.h>
#include <qfiledialog.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qfileinfo.h>
#include <qaccel.h>

#include <cmath>	// pour abs

//	****
//	Log : lower/raise
//
//		Est cache si :
//			- Timer passe  zro.
//			- Timer  zro au dmarrage de la simulation.
//			- sur pause off si timer  zro
//
//		Est rendu visible :
//			- Si trace de log et timer ne vaut pas zro.
//			- Si pause on *et* timer vaut zro, si timer ne vaut pas
//			  zro c'est l'utilisateur qui  fait passer la log en
//			  arrire plan.
//			- Si timer passe de zro a positif.
//	****

namespace
{
	enum { NoTimer = -1, TimerPause = -2 };
	const QString	GB_labelBtnPauseOn( "S&ur" );
	const uint		GB_metersPerPixel	= 10;
	const uint		GB_secondesPerTop	= 1;
	const double	GB_minutesPerTop	= GB_secondesPerTop / 60.0;
	const double	GB_hoursPerTop		= GB_minutesPerTop  / 60.0;

	const int GB_intervalTimerEdited	=  900;
	const int GB_intervalTimerLine		= 2000;
	const int GB_intervalTimerScript	=  100;

	inline double elapsedMinutes( int tops )
	{
		return tops * GB_minutesPerTop;
	}

	inline double elapsedHours( int tops )
	{
		return tops * GB_hoursPerTop;
	}

	inline int pixelsToMeters( int totalPixels )
	{
		return totalPixels * GB_metersPerPixel;
	}

	inline double Kmeters( int distance )
	{
		return distance / 1000.0;
	}

	QDateTime	GB_lastModified;

	inline QString strTrajectoire( const Pixel& po, const Pixel& pd )
	{
		return "Trajectoire " +	outPutPixel(po) + " -> " + outPutPixel(pd);
	}

};

TrackSimulator::TrackSimulator( QWidget* parent, const char* name )
:	 TrackMainWindow( TrackMainWindow::TrackSimulator, parent, name )
	,f1Stream_( &f1File_ )
	,startCoords_(-1)
	,idTimer_(NoTimer)
	,prevPixel_(NULL)
	,pauseOn_(NULL)
	,pauseOnTrace_(false)
	,idTimerLine_(-1)
	,oriLine_(NULL)
	,idTimerEdited_(-1)
	,editState_(Not)
	,idTimerScript_(-1)
{
	setIconText( "Simulateur: " + iconText() );
	winResults_ = new DialogResults( this );
	winResults_->move( config_.windowResultsX(), config_.windowResultsY() );

	const Icons& icons = Icons::instance();
	//	---
	//	
	//	---
	btnScript_ = new QPushButton( icons.script(), "&Script", parentTopBar() );
	btnScript_->setFlat( true );
	btnScript_->setToggleButton( true );
	btnScript_->setOn( Script::instance().on() );
	connect( btnScript_, SIGNAL(toggled(bool)), SLOT(script(bool)) );
	addTopBar( btnScript_ );

	btnLoadDriver_ =
		new QPushButton( icons.fileOpen(), "&Pilote", parentTopBar() );
	btnLoadDriver_->setFlat( true );
	addTopBar( btnLoadDriver_ );
	connect( btnLoadDriver_, SIGNAL(clicked()), SLOT(loadDriver()) );

	//	---
	//	Bouttons de contrle de la simulations
	QHBoxLayout* ctrlLayout = new QHBoxLayout;

	btnGo_ = new QPushButton( icons.start(), "Go", parentTopBar() );
	btnGo_->setAccel( Key_Return );
	btnGo_->setFlat( true );
	ctrlLayout->addWidget( btnGo_, 1 );
	connect( btnGo_, SIGNAL(clicked()), SLOT(go()) );

	btnPause_ = new QPushButton( icons.pause(), "Pause", parentTopBar() );
	btnPause_->setAccel( Key_Space );
	btnPause_->setFlat( true );
	btnPause_->setToggleButton( true );
	ctrlLayout->addWidget( btnPause_, 1 );
	connect( btnPause_, SIGNAL(toggled(bool)), SLOT(pause(bool)) );

	btnPauseOn_ = new QPushButton( GB_labelBtnPauseOn, parentTopBar() );
	btnPauseOn_->setFlat( true );
	ctrlLayout->addWidget( btnPauseOn_, 1 );
	connect( btnPauseOn_, SIGNAL(clicked()), SLOT(pauseOn()) );

	btnStep_ = new QPushButton( icons.step(), "Pas", parentTopBar() );
	btnStep_->setAccel( Key_Right );
	btnStep_->setFlat( true );
	btnStep_->setAutoRepeat( true );
	ctrlLayout->addWidget( btnStep_, 1 );
	connect( btnStep_, SIGNAL(clicked()), SLOT(step()) );
	btnStep_->setDisabled( true );

	btnStop_ = new QPushButton( icons.stop(), "Stop", parentTopBar() );
	btnStop_->setAccel( Key_End );
	btnStop_->setFlat( true );
	ctrlLayout->addWidget( btnStop_, 1 );
	connect( btnStop_, SIGNAL(clicked()), SLOT(stop()) );
	btnStop_->setDisabled( true );

	addTopBar( ctrlLayout );
	
	addSeparatorTopBar();

	//	---
	//
	//	---
	sliderTimerInterval_ =
			new QSlider( 0, Config::timerMax, Config::timerStep,
						config_.timer(), QSlider::Horizontal, parentTopBar() );
	sliderTimerInterval_->setTickmarks( QSlider::Both );
	connect( sliderTimerInterval_,	SIGNAL(valueChanged(int)),
									SLOT(timerIntervalChanged(int)) );
	QAccel* accelsSlider = new QAccel( this );
	accelsSlider->connectItem(
		accelsSlider->insertItem(Key_Plus), sliderTimerInterval_, 
															SLOT(addStep()) );
	accelsSlider->connectItem(
		accelsSlider->insertItem(Key_Minus), sliderTimerInterval_,
														SLOT(subtractStep()) );
	QToolTip::add( sliderTimerInterval_,
					" droite temps rel.\n"
					" gauche animation la plus rapide.\n"
					"Chaque graduation reprsente "
						+ QString::number(Config::timerStep) + " ms."
					);

	addTopBar( sliderTimerInterval_ );

	addSeparatorTopBar();

	//	----
	//
	//	----
	lcdSpeed_ = new QLCDNumber( 3, parentTopBar() );
	lcdSpeed_->setSegmentStyle( QLCDNumber::Flat );
	lcdSpeed_->setFrameStyle( QFrame::StyledPanel | QFrame::Raised );
	QPalette palLCD = lcdSpeed_->palette();
	palLCD.setColor( QPalette::Active, QColorGroup::Text, Qt::green );
	palLCD.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );
	palLCD.setColor( QPalette::Active, QColorGroup::Base, Qt::black );
	palLCD.setColor( QPalette::Active, QColorGroup::Background, Qt::black );

	palLCD.setColor( QPalette::Inactive, QColorGroup::Text, Qt::green );
	palLCD.setColor( QPalette::Inactive, QColorGroup::Foreground, Qt::green );
	palLCD.setColor( QPalette::Inactive, QColorGroup::Base, Qt::black );
	palLCD.setColor( QPalette::Inactive, QColorGroup::Background, Qt::black );

	lcdSpeed_->setPalette( palLCD );

	addTopBar( lcdSpeed_ );
	addTopBar( new QLabel( "km/h", parentTopBar() ) );

	lcdDistance_ = new QLCDNumber( 5, parentTopBar() );
	lcdDistance_->setSegmentStyle( QLCDNumber::Flat );
	lcdDistance_->setFrameStyle( QFrame::StyledPanel | QFrame::Raised );
	addTopBar( lcdDistance_ );
	addTopBar( new QLabel( "m", parentTopBar() ) );

	addSeparatorTopBar();

	QToolButton* btnQuit = new QToolButton( parentTopBar() );
	btnQuit->setTextLabel( "Fermer" );
	btnQuit->setAccel( ALT+Key_F );
	btnQuit->setIconSet( icons.exit() );
	addTopBar( btnQuit );
	connect( btnQuit, SIGNAL(clicked()), SLOT(close()) );
	//	----

	connect( widgetTrack_, SIGNAL(mouseLeftPressPixel(QMouseEvent*,Pixel&)),
						SLOT(mouseLeftPressPixel(QMouseEvent*,Pixel&)) );
	connect( widgetTrack_, SIGNAL(mouseMouveOnPixel(Pixel&)),
						SLOT(mouseMouveOnPixel(Pixel&)) );
	connect( widgetTrack_, SIGNAL(mouseLeftReleasePixel(QMouseEvent*,Pixel&)),
						SLOT(mouseLeftReleasedPixel(QMouseEvent*,Pixel&)) );
}

//	+++++++++++++++++++++++++++++++++++++++++++++++++
//	+++++++++++++++++++++++++++++++++++++++++++++++++
void TrackSimulator::trackEdited( const QString& track, EditState es )
{
	if ( track == trackName_ )
	{
		switch ( es )
		{
		case Not:
			assert( false );
			break;

		case Modified:
			if ( idTimerEdited_ == -1 )
			{
				editState_ = Modified;
				idTimerEdited_ = startTimer( GB_intervalTimerEdited );
			}
			break;

		case Closed:
			if ( editState_ == Modified )
			{
				assert( idTimerEdited_ != -1 );
				editState_ = Not;
				killTimer( idTimerEdited_ );
				idTimerEdited_ = -1;
				setCaption( trackName_ );
			}
			break;

		case Saved:
			if ( editState_ == Modified )
				editState_ = Saved;
			break;
		}
	}
}

void TrackSimulator::cleanLine()
{
	assert( idTimerLine_ != -1 );
	killTimer( idTimerLine_ );
	idTimerLine_ = -1;
	oriLine_ = NULL;
	widgetTrack_->cleanSelect( false );
}

//	++++++++++++++++++++++++++++++
//	++++ Actions de la souris ++++
//	++++++++++++++++++++++++++++++
void TrackSimulator::mouseLeftPressPixel( QMouseEvent* me, Pixel& pixel )
{
	statusBar_.message( outPutPixel( pixel ) );
	if ( ! (me->state() & ControlButton) )
	{
		if ( oriLine_ != NULL )
			cleanLine();
		oriLine_ = &pixel;
	}
}

void TrackSimulator::mouseMouveOnPixel( Pixel& pixel )
{
	if ( oriLine_ != NULL )
	{
		widgetTrack_->join( Qt::black, *oriLine_, pixel );
		const uint distanceX = std::abs(pixel.x() - oriLine_->x());
		const uint distanceY = std::abs(pixel.y() - oriLine_->y());
		statusBar_.message(
				strTrajectoire( *oriLine_, pixel )
			+	" distance (x,y) " + outPutPoint(distanceX,distanceY) );
	}
}

void TrackSimulator::mouseLeftReleasedPixel( QMouseEvent* me, Pixel& pixel )
{
	if ( oriLine_ != NULL )
	{
		idTimerLine_ = startTimer( GB_intervalTimerLine );
	}
	else
	{
		QString msg = outPutPixel( pixel );
		if ( me->state() & ControlButton )
		{
			pauseOn_ = &pixel;
			btnPauseOn_->setText( GB_labelBtnPauseOn + ' ' + 
										outPutPoint( pixel.x(), pixel.y() ) );
			msg = "Pause sur le pixel " + msg;
		}

		statusBar_.message( msg );
	}
}

//	+++++++++++++++++++++++++++++++++++++++++++++++++
//	+++++++++++++++++++++++++++++++++++++++++++++++++
void TrackSimulator::initResults()
{
	winResults_->setTime( elapsedMinutes(topTimer_) );
	winResults_->setDistance( static_cast<int>(totalDistanceMeters_) );
	winResults_->setSpeedMax( maxSpeed_ );
	winResults_->setSpeedMean( Kmeters(totalDistanceMeters_) / elapsedHours(topTimer_) );
}

//	+++++++++++++++++++++++++++++++++++++++++++++++++
//	++++	Fonctions relativent  la fentre de Log.
//	+++++++++++++++++++++++++++++++++++++++++++++++++
void TrackSimulator::lowerRaiseLogDebug()
{
	if ( ! logDebug_ )
		return;

	// La log est active si :
	// 		le timer > 0
	// 	et	que pause est off
	logDebug_->setAutoUpdate(
					sliderTimerInterval_->value() != 0 || btnPause_->isOn() );

	// Cache la log si
	// 		le timer == 0
	// 	et	pause off
	if ( sliderTimerInterval_->value() == 0  && ! btnPause_->isOn() )
		logDebug_->lower();
	else
	{
		logDebug_->show();
		logDebug_->raise();
	}
}

void TrackSimulator::appendLogDebug( const QString& line )
{
	if ( ! logDebug_ )
	{
		logDebug_ = new Log( this );
		QAccel* accel = new QAccel( logDebug_ );
		accel->connectItem( accel->insertItem(CTRL+Key_F1),
													this, SIGNAL(helpLog()) );
		logDebug_->setGeometry( config_.windoLogX(), config_.windoLogY(),
									config_.windoLogW(), config_.windoLogH() );
		lowerRaiseLogDebug();
	}
	logDebug_->append( line );
}

//	+++++++++++++++++++++++++++++++++++++++++++++++++++++
//	++++	Fonctions relativent  l'excution du script.
//	+++++++++++++++++++++++++++++++++++++++++++++++++++++
void TrackSimulator::printErrorScript()
{
	QFile logS( Script::logFile );
	if ( logS.open( IO_ReadOnly ) )
	{
		const Script& script = Script::instance();
		appendLogDebug( "" );
		appendLogDebug( "Code de retour du script : "
										+ QString::number(script.exitCode()) );
		if ( script.exitCode() == -1 )
			appendLogDebug( script.error() );

		appendLogDebug( "Canal d'erreur du script" );
		QTextStream s( &logS );
		appendLogDebug( s.read() );
		logDebug_->forceUpdate();
		logDebug_->raise();
	}
}

void TrackSimulator::endScript()
{
	lowerRaiseLogDebug();

	const QFileInfo fi( f1File_ );
	if ( fi.lastModified() != GB_lastModified )
	{
		f1File_.close();
		GB_lastModified = fi.lastModified();
		if ( ! loadFileDriver() )
			return;
		statusBar_.message( "Le fichier " + f1File_.name()
													+ " a t recharg." );
	}

	start();
}

namespace
{
	int	GB_tods;
};

void TrackSimulator::printExecScript()
{
	QString strTods; strTods.fill( '*', GB_tods );
	
	statusBar_.message( "Excution du script " + strTods );
	if ( ++GB_tods == 120 )
		GB_tods = 3;
}

void TrackSimulator::execScript()
{
	Script& execScript = Script::instance();
	if ( execScript.on() )
	{
		GB_tods = 3;
		if ( ! execScript.exec() )
		{
			printErrorScript();
			statusBar_.message( "chec excution du script." );
		}
		else
			idTimerScript_ = startTimer( GB_intervalTimerScript );
	}
	else
		endScript();
}

void TrackSimulator::script( bool on )
{
	Script& execScript = Script::instance();
	execScript.setOn( on );
	if ( on )
	{
		DialogScript ds( &execScript, this, 0 );
		const QPoint topLeft( btnScript_->x(),
									btnScript_->y()+ btnScript_->height() );
		ds.move( mapToParent( topLeft ) );

		if ( ds.exec() == 0 )
			execScript.setOn( false );

		btnScript_->blockSignals( true );
		btnScript_->setOn( execScript.on() );
		btnScript_->blockSignals( false );
	}
}

//	+++++++++++++++++++++++++++++++++++++++++++++++++++++
//	++++ Contrles sur le droulement de l'animation ++++
//	+++++++++++++++++++++++++++++++++++++++++++++++++++++
void TrackSimulator::go()
{
	if ( logDebug_ )
		logDebug_->clear();

	if ( startCoords_ == -1 && ! loadDriver() )
		return;

	execScript();
}

void TrackSimulator::pause( bool pauseOn )
{
	btnStep_->setDisabled( ! pauseOn );
	switch ( pauseOn )
	{
	case true:
		if ( idTimer_ != NoTimer )
		{
			killTimer( idTimer_ );
			idTimer_ = TimerPause;
		}

		lowerRaiseLogDebug();
		if ( logDebug_ )
			logDebug_->forceUpdate();

		statusBar_.message( "Pause..." );
		break;

	case false:
		// Le boutton Pause peut tre 'click' mme si aucune piste ou
		// aucun pilote n'est charg.
		if ( idTimer_ == TimerPause )
		{
			assert( startCoords_ != -1 );
			idTimer_ = startTimer( sliderTimerInterval_->value() );
			lowerRaiseLogDebug();
			statusBar_.message( "Running..." );
		}
	}
}

void TrackSimulator::pauseOn()
{
	int x, y;
	if ( pauseOn_ == NULL )
	{
		if ( pauseOnTrace_ )
		{
			x = -2;
			y = 0;
		}
		else
		{
			x = 40;
			y = 4;
		}
	}
	else
	{
		x = pauseOn_->x();
		y = pauseOn_->y();
	}
	
	DialogPauseOn dpo( x, y, this );
	const QPoint topLeft( btnPauseOn_->x(),
									btnPauseOn_->y()+ btnPauseOn_->height() );
	dpo.move( mapToParent( topLeft ) );

	if ( dpo.exec() )
	{
		if ( dpo.pauseX() == -1 )
		{
			pauseOnTrace_ = true;
			btnPauseOn_->setText( GB_labelBtnPauseOn + " traces" );
		}
		else
		{
			pauseOnTrace_ = false;
			pauseOn_ =
			&widgetTrack_->pixel( Dim::numPixel( dpo.pauseX(), dpo.pauseY() ) );
			btnPauseOn_->setText( GB_labelBtnPauseOn + ' ' +
								outPutPoint( pauseOn_->x(), pauseOn_->y() ) );
		}
	}
	else
	{
		pauseOn_ = NULL;
		btnPauseOn_->setText( GB_labelBtnPauseOn );
	}
}

void TrackSimulator::step()
{
	if ( startCoords_ != -1 && idTimer_ == TimerPause )
	{
		newTop();
		if ( logDebug_ )
			logDebug_->forceUpdate();
	}
}

void TrackSimulator::stop()
{
	if ( idTimer_ != TimerPause )
	{
		assert( idTimer_ != NoTimer );
		killTimer( idTimer_ );
	}

	idTimer_ = NoTimer;

	// On se positionne  la fin pour que read n'est pas grand chose  lire,
	// surtout quand le pilote logiciel produit des fichiers de plusieurs
	// 100Mo :))
	f1File_.at( f1File_.size() );
	// On vide le stream, sinon des problmes apparaissent pour la lecture
	// d'un autre fichier (la position dans le stream est invalide)
	f1Stream_.read();

	lcdSpeed_->display( 0 );

	btnLoadDriver_->setDisabled( false );
	btnGo_->setDisabled( false );
	btnScript_->setDisabled( false );
	btnStop_->setDisabled( true );
	btnStep_->setDisabled( true );

	if ( pauseOn_ != NULL && btnPause_->isOn() )
		btnPause_->setOn( false );

	if ( logDebug_ )
	{
		if ( ! logDebug_->autoUpdate() )
			logDebug_->setAutoUpdate( true );

		logDebug_->forceUpdate();
		logDebug_->show();
		logDebug_->raise();
	}

	Script& execScript = Script::instance();
	if ( execScript.on() )
	{
		printErrorScript();
		statusBar_.message(	execScript.error() != "" 
						?	execScript.error()
						:	QString( "Le script c'est termin normallement." )
						);
	}
}

void TrackSimulator::timerIntervalChanged( int value )
{
	if ( idTimer_ != NoTimer && idTimer_ != TimerPause )
	{
		killTimer( idTimer_ );
		idTimer_ = startTimer( value );
		lowerRaiseLogDebug();
	}
}

//	+++++++++++++++++++++++++++++++++++++++++++++++++
//	+++++++++++++++++++++++++++++++++++++++++++++++++
bool TrackSimulator::loadTrack()
{
	if ( widgetTrack_->load( trackName_, false ) )
	{
		QString msg;
		
		if ( widgetTrack_->colorUsed( config_.driverColor() ) )
			msg = "La couleur du pilote est dj utilise pour la piste.";

		if ( widgetTrack_->colorUsed( config_.traceColor() ) )
			msg += "\nLa couleur de 'trace' est dj utilise pour la piste.";

		if ( ! msg.isEmpty() )
		{
			QMessageBox::critical( this, "CONFIG INVALIDE",
				msg + "\n\nCorriger la configuration avant de continuer !" );
		}
		else
			return true;
	}

	return false;
}

bool TrackSimulator::loadFileDriver()
{
	if ( f1File_.open( IO_ReadOnly ) )
	{
		//	Ligne 1 : Nom du pilote.
		winResults_->setDriver( f1Stream_.readLine() );
		winResults_->setInfo( "Prt..." );
		winResults_->show();

		//	Ligne 2 : Nom du fichier de description de la piste.
		QString piste = f1Stream_.readLine();
		if ( ! QFile::exists( piste ) )
			piste =	config_.pathTracks() + '/' + piste;

		bool killEditTimer = idTimerEdited_ != -1;
		if ( killEditTimer )
		{
			switch ( editState_ )
			{
			case Not:		// impossible
			case Closed:	// impossible
				assert( false );
				break;

			case Modified:
				killEditTimer = piste != trackName_;
				break;

			case Saved:
				killEditTimer = true;
			}

			if ( killEditTimer )
			{
				killTimer( idTimerEdited_ );
				idTimerEdited_ = -1;
				editState_ = Not;
			}
		}

		trackName_ = piste;
		setCaption( trackName_ );

		startCoords_ = f1File_.at();
		if ( loadTrack() )
		{
			statusBar_.message( "Pilote charg." );
			return true;
		}
		else
			setCaption( "INVALIDE : " + piste );
	}

	return false;
}

bool TrackSimulator::loadDriver()
{
	f1File_.close();
	const QString file =
			QFileDialog::getOpenFileName( config_.pathDrivers(),
				"Log Pilote (*."+ext::result+")", NULL, 0, "Charger Pilote" );

	if ( ! file.isEmpty() )
	{
		if ( ! f1File_.name().isEmpty() )	// change de fichier f1 donc on
			btnScript_->setOn( false );		// annule l'excution du script.

		delete logDebug_;	// pas de NULL ==> QGuardedPtr

		f1File_.setName( file );
		GB_lastModified = QFileInfo( file ).lastModified();
		return loadFileDriver();
	}

	return false;
}

void TrackSimulator::start()
{
	assert( startCoords_ != -1 );

	if ( idTimer_ != NoTimer && ! btnPause_->isOn() )
	{
		killTimer( idTimer_ );
		idTimer_ = NoTimer;
	}

	if ( ! loadTrack() )
		return;

	f1File_.at( startCoords_ );
	int x, y;
	char sep;
	f1Stream_ >> x >> sep >> y;
	if ( x != 40 && y != 4 )
	{
		QMessageBox::critical( this, "FICHIER INVALIDE",
				"La position de dpart doit tre en [40,4],\n"
				"elle est  la position " + outPutPoint(x,y) );
		return;
	}

	const int numPixel = Dim::numPixel( x, y );
	prevPixel_ = &widgetTrack_->pixel( numPixel );

	if ( btnPause_->isOn() )
	{
		idTimer_ = TimerPause;
		btnStep_->setDisabled( false );
	}

	btnLoadDriver_->setDisabled( true );
	btnGo_->setDisabled( true );
	btnScript_->setDisabled( true );
	btnStop_->setDisabled( false );

	widgetTrack_->paintPixel( config_.driverColor(), numPixel );

	topTimer_ = totalDistanceMeters_ = 0;
	maxSpeed_ = 0;
	winResults_->setCaption( "Course en cours..." );
	winResults_->setInfo( "..." );
	initResults();
	if ( idTimer_ != TimerPause )
	{
		idTimer_ = startTimer( sliderTimerInterval_->value() );
		statusBar_.message( "Course en cours..." );
	}
	else
		statusBar_.message( "Prt..." );
}

//	++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	++++	Fonctions de gestion du dplacement du pixel F1.
//	++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void TrackSimulator::collisionAt( int pixelErr, int numPixel )
{
	stop();

	const int crossWidth = config_.crossSize();
				
	int fxPixel = pixelErr - crossWidth / 2;
	int lxPixel = pixelErr + crossWidth / 2;
				
	do widgetTrack_->paintPixel( config_.crossColor(), fxPixel );
	while ( ++fxPixel <= lxPixel );

	fxPixel = pixelErr - (Dim::maxX*crossWidth);
	if ( fxPixel < 0 )
		fxPixel = pixelErr;
	lxPixel = pixelErr + (Dim::maxX*crossWidth);
	do 
	{
		widgetTrack_->paintPixel( config_.crossColor(), fxPixel );
		fxPixel += Dim::maxX;
	} while ( fxPixel <= lxPixel );

	const QString msg1 = "Collision au pixel " 
							+ outPutPixel( widgetTrack_->pixel( pixelErr ) );
	const QString msg2 =
				strTrajectoire( *prevPixel_, widgetTrack_->pixel( numPixel ) );

	statusBar_.message( msg1 + ' ' + msg2 );
	winResults_->setInfo( msg1 + '\n' + msg2 );
	winResults_->setCaption( "SORTIE DE PISTE" );
	initResults();
			
	if ( ! winResults_->isVisible() )
		winResults_->show();
	else
		winResults_->raise();
}

void TrackSimulator::moveAt( int numPixel )
{
	const Pixel& pixel		= widgetTrack_->pixel( numPixel );

	const uint distanceX	= std::abs(pixel.x() - prevPixel_->x());
	const uint distanceY	= std::abs(pixel.y() - prevPixel_->y());
	const uint meters		= pixelsToMeters( distanceX + distanceY );
	const double speedKH	= Kmeters(meters) / (GB_hoursPerTop);
	lcdSpeed_->display( speedKH );

	totalDistanceMeters_ += meters;
	lcdDistance_->display( static_cast<int>(totalDistanceMeters_) );

	if ( speedKH > maxSpeed_ )
		maxSpeed_ = speedKH;

	if ( winResults_->isVisible() )
		initResults();

	if ( btnPause_->isOn() )
	{
		const QString m = strTrajectoire( *prevPixel_, pixel )
					+ ", distance (x,y) = " + outPutPoint(distanceX,distanceY)
					+ " = " + QString::number(meters) + "m"
					;
		statusBar_.message(m);
	}

	prevPixel_ = &pixel;
	if ( pauseOn_ != NULL && pixel == *pauseOn_ )
	{
		btnPause_->setOn( true );
		statusBar_.message(
						"Bascule sur pause au pixel " + outPutPixel( pixel ) );
	}
}

void TrackSimulator::drawFinishLine( bool WBcolor )
{
	for ( int ipixel=340; ipixel<=940; ipixel+=Dim::maxX, WBcolor=!WBcolor )
		widgetTrack_->shadowPixel( (WBcolor ? Qt::black : Qt::white), ipixel );
}

void TrackSimulator::newTop()
{
	assert( startCoords_ != -1 );

	int x, y;
	do
	{
		f1Stream_ >> x;
		switch ( x )
		{
		case 0:	// le pixel 0 ne peut tre utilis.
			if ( f1Stream_.atEnd() )
			{
				stop();
				winResults_->setCaption( "VAINQUEUR" );
				winResults_->setInfo( "Bravo" );
				initResults();
				statusBar_.message( "Finie." );
				if ( ! winResults_->isVisible() )
					winResults_->show();
				else
					winResults_->raise();
			}
			else
			{
				QString l; f1Stream_ >> l;
				stop();
				appendLogDebug( "Erreur de lecture '" + l + "'." );
			}
			return;

		case -1:
			appendLogDebug( f1Stream_.readLine() );
			if ( pauseOnTrace_ )
				btnPause_->setOn( true );
			break;

		case -2:
			{
				int nPixel;
				f1Stream_ >> nPixel;
				if ( Dim::pixelValid( nPixel ) )
					widgetTrack_->paintPixel( config_.traceColor(), nPixel );
				else
				{
					btnPause_->setOn( true );
					const QString msg = "***> TRACE: pixel "
									+ QString::number(nPixel) + " invalide !";
					appendLogDebug( msg );
					statusBar_.message( msg );
					return;
				}
			}
			break;

		default:
			{
				char sep;
				f1Stream_ >> sep >> y;
			}
		}
	} while ( x == -1 || x == -2 );

	++topTimer_;
	const int numPixel = Dim::numPixel(x,y);
	if ( ! Dim::pixelValid( numPixel ) )
	{
		stop();
		QMessageBox::critical( this, "DEPLACEMENT",
			"Le dplacement " + outPutPoint(x,y) + " est invalide." );
		return;
	}
		
	const int pixelErr = 
		widgetTrack_->moveCar( config_.driverColor(), config_.traceColor(), 
								*prevPixel_, widgetTrack_->pixel( numPixel ) );

	if ( pixelErr != -1 )
		collisionAt( pixelErr, numPixel );
	else
	{
		moveAt( numPixel );
		if ( topTimer_%7 == 0 )
			drawFinishLine( topTimer_%2 == 0 );
	}
}

void TrackSimulator::timerEvent( QTimerEvent* te )
{
	if ( te->timerId() == idTimerEdited_ )
	{
		static int sens = 1;
		setCaption( "|-"[sens] + trackName_ + " en dition" );
		sens = ! sens;
	}
	else if ( te->timerId() == idTimerScript_ )
	{
		Script& execScript = Script::instance();
		if ( ! execScript.running() )
		{
			killTimer( idTimerScript_ );
			idTimerScript_ = -1;
			switch( execScript.exitCode() )
			{
			case -1:
			case  3:
				printErrorScript();
				statusBar_.message( "Script error" );
				break;

			default:
				endScript();
			}
		}
		else
			printExecScript();
	}
	else if ( te->timerId() == idTimerLine_ )
		cleanLine();
	else 
		newTop();
}

void TrackSimulator::keyPressEvent( QKeyEvent* ke )
{
	if ( idTimerScript_ != -1 && 
					ke->state() == ControlButton && ke->key() == Key_C )
	{
		statusBar_.message( "Arrt brutal de l'excution du script." );
		Script::instance().kill();
		ke->accept();
	}
	else
		TrackMainWindow::keyPressEvent( ke );
}

TrackSimulator::~TrackSimulator()
{
	config_.setTrackSimulatorXY( x(), y() );
	if ( winResults_ )
		config_.setWindowResultsXY( winResults_->x(), winResults_->y() );
	if ( logDebug_ )
		config_.setWindowLog( logDebug_->x(), logDebug_->y(),
									logDebug_->width(), logDebug_->height() );
}

